'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.mapForReact = mapForReact;
exports.mapToRelative = mapToRelative;
exports.asAbsolute = asAbsolute;
exports.mapModule = mapModule;
var path = require('path');

var fs = require('fs');

var dirContentsMap = {};

function getStateOptions(s) {
  if (s.opts) {
    if (Array.isArray(s.opts)) {
      return { map: s.opts };
    } else if (!s.opts.map && s.opts.expose) {
      return { map: [s.opts] };
    }

    return s.opts;
  }

  return {};
}

function getRootPath(s) {
  if (s.root) {
    return s.root;
  } else if (s.opts) {
    return s.opts.root || '';
  }

  return '';
}

function createFilesMap(state) {
  var result = {};
  var opts = getStateOptions(state);
  var rootPath = getRootPath(opts);
  var map = opts.map;

  map.forEach(function (m) {
    var expose = m.expose || undefined;
    var src = m.src || undefined;

    if (expose) {
      if (typeof src === 'string') {
        if (!src) {
          // Consider non existing paths as referencing to the root path
          return rootPath;
        }

        result[expose] = path.join(rootPath, src);
      }
    }

    return null;
  });

  return result;
}

function resolve(filename) {
  if (path.isAbsolute(filename)) return filename;
  return path.resolve(process.cwd(), filename);
}

function toPosixPath(modulePath) {
  return modulePath.replace(/\\/g, '/');
}

function toReactPath(base, fileList, o) {
  // TODO: Check if the current file is a directory, if it is check the logic on index.js
  var os = (o || process.env.TARGET_PLATFORM || '').toLowerCase();

  // throw new Error(`${os} | ${base} | ${fileList}`);
  // Check for mobile substitutions first
  if (['mobile', 'ios', 'android', 'windows'].indexOf(os) > -1) {
    // Check for a mobile file first
    if (fileList.indexOf(base + '.mobile.js') > -1) {
      return base + '.mobile';
    }

    // Check for OS specific file
    if (fileList.indexOf(base + '.' + os + '.js') > -1) {
      return base + '.' + os;
    }

    // Fallback on normal file (web/desktop) for passthrow files
    return toReactPath(base, fileList, 'desktop');
  } else if (os === 'desktop') {
    // Check for desktop only files
    if (fileList.indexOf(base + '.desktop.js') > -1) {
      return base + '.desktop';
    }

    // Fallback on web
    return toReactPath(base, fileList, 'web');
  } else if (os === 'web') {
    if (fileList.indexOf(base + '.web.js') > -1) {
      return base + '.web';
    }
  }

  // Suffixless files
  if (fileList.indexOf(base + '.js') > -1) {
    return '' + base;
  }

  return base;
}

function mapForReact(moduleMapped) {
  // moduleMapped =
  var base = path.dirname(moduleMapped);

  // Index files in destination directory once
  if (!dirContentsMap[base]) {
    dirContentsMap[base] = fs.readdirSync(base).map(function (v) {
      return v.toLowerCase();
    });
  }

  // Fix mapped module path
  var newFile = toReactPath(path.basename(moduleMapped), dirContentsMap[base]);

  return base + '/' + newFile;
}

function mapToRelative(currentFile, module, options) {
  var o = options || {};
  var reactOsFileInfer = !!o.react;
  var from = path.dirname(currentFile);
  var to = path.normalize(module);

  from = resolve(from);
  to = resolve(to);

  var moduleMapped = path.relative(from, to);

  moduleMapped = toPosixPath(moduleMapped);

  // Support npm modules instead of directories
  if (moduleMapped.indexOf('npm:') !== -1) {
    var _moduleMapped$split = moduleMapped.split('npm:');

    var _moduleMapped$split2 = _slicedToArray(_moduleMapped$split, 2);

    var npmModuleName = _moduleMapped$split2[1];

    return npmModuleName;
  }

  if (moduleMapped[0] !== '.') moduleMapped = './' + moduleMapped;

  // Support React-Native specific require rewrites
  if (reactOsFileInfer) {
    return mapForReact(moduleMapped);
  }

  return moduleMapped;
}

function asAbsolute(currentFile, module, options) {
  var reactOsFileInfer = !!options.react;
  var m = module || options.root;

  if (!path.isAbsolute(m)) {
    m = path.join(options.root, m);
  }

  // throw new Error(`${JSON.stringify(options, null, 2)}`);

  if (reactOsFileInfer) {
    return mapForReact(m);
  }

  return m;
}

function mapModule(source, file, filesMap, state) {
  var moduleSplit = source.split('/');
  var opts = getStateOptions(state);
  var absolutePaths = !!getRootPath(opts);

  var src = void 0;
  while (moduleSplit.length) {
    var m = moduleSplit.join('/');

    if ({}.hasOwnProperty.call(filesMap, m)) {
      src = filesMap[m];
      break;
    }

    moduleSplit.pop();
  }

  if (absolutePaths) {
    if (!src && source[0] !== '.') {
      // Do not break node_modules required by name
      return null;
    }

    if (path.isAbsolute(source)) {
      // After the node is replaced the visitor will be called again.
      // Without this condition these functions will generate a circular loop.
      return null;
    }

    return asAbsolute(file, moduleSplit.length ? source.replace(moduleSplit.join('/'), src) : path.join(opts.root, source), opts);
  }

  if (!moduleSplit.length) {
    // no mapping available
    return null;
  }

  return mapToRelative(file, source.replace(moduleSplit.join('/'), src), opts);
}

exports.default = function (_ref) {
  var t = _ref.types;

  function transformRequireCall(nodePath, state) {
    if (!t.isIdentifier(nodePath.node.callee, { name: 'require' }) && !(t.isMemberExpression(nodePath.node.callee) && t.isIdentifier(nodePath.node.callee.object, { name: 'require' }))) {
      return;
    }

    var moduleArg = nodePath.node.arguments[0];
    if (moduleArg && moduleArg.type === 'StringLiteral') {
      var filesMap = createFilesMap(state);

      var modulePath = mapModule(moduleArg.value, state.file.opts.filename, filesMap, state);
      if (modulePath) {
        nodePath.replaceWith(t.callExpression(nodePath.node.callee, [t.stringLiteral(modulePath)]));
      }
    }
  }

  function transformImportCall(nodePath, state) {
    var moduleArg = nodePath.node.source;
    if (moduleArg && moduleArg.type === 'StringLiteral') {
      var filesMap = createFilesMap(state);

      var modulePath = mapModule(moduleArg.value, state.file.opts.filename, filesMap, state);
      if (modulePath) {
        nodePath.replaceWith(t.importDeclaration(nodePath.node.specifiers, t.stringLiteral(modulePath)));
      }
    }
  }

  return {
    visitor: {
      CallExpression: {
        exit: function exit(nodePath, state) {
          return transformRequireCall(nodePath, state);
        }
      },
      ImportDeclaration: {
        exit: function exit(nodePath, state) {
          return transformImportCall(nodePath, state);
        }
      }
    }
  };
};